@@ -72,13 +72,14 @@ class Binary_Dice_Loss(nn.Module):
 
 
 class Multi_Dice_Loss(nn.Module):
-    def __init__(self, num_classes, smooth = 1.0, background_index = 0):
+    def __init__(self, num_classes, smooth = 0.0, epsilon = 1e-8, background_index = 0):
         super(Multi_Dice_Loss, self).__init__()
         self.smooth = smooth
         self.background_index = background_index
         self.num_classes = num_classes
         self.mask = (torch.arange(0, num_classes) != self.background_index)
         self.reduction = 'none'
+        self.epsilon = epsilon
 
     def forward(self, output, target):
         #output shape => (n_bs, n_classes, -1)
@@ -93,7 +94,7 @@ class Multi_Dice_Loss(nn.Module):
         intersection = (output * target).sum((-1,-2)) #각 class 별 intersection score를 합함.
         union = output.sum((-1,-2)) + target.sum((-1,-2)) #각 class 별 union score를 합함.
 
-        dice = 2.0 * (intersection + self.smooth) / (union + self.smooth)
+        dice = 2.0 * (intersection + self.smooth) / (union + self.smooth + self.epsilon)
         
         return 1 - dice
 
