@@ -48,7 +48,42 @@ class ConvDropoutNormNonlin(nn.Module):
         return self.lrelu(self.instnorm(x))
 
 
-class Deep_Supervision_UNet3d_V2(nn.Module):
+
+class DeepSuperVisionUNet2dot5(nn.Module):
+    def __init__(self, n_modalities = 1, depth = 256, fianl_output_channels = 2, patch_size = (256,512,512)):
+        conv_kwargs1 = {'kernel_size' : 3, 'padding' : 1, 'stride' : 1}
+        encoder_channel1 = 64
+        encoder_channel2 = encoder_channel1 * 2
+        encoder_channel3 = encoder_channel2 * 2
+        encoder_channel4 = encoder_channel3 * 2
+        encoder_channel5 = encoder_channel4 * 2
+
+        self.encoder1 = nn.Sequential(
+            ConvDropoutNormNonlin(n_modalities * depth, encoder_channel1, conv_kwargs= conv_kwargs1, conv_op = nn.Conv2d, norm_op = nn.InstanceNorm2d, dropout_op = nn.Dropout2d),
+            ConvDropoutNormNonlin(encoder_channel1, encoder_channel1, conv_kwargs = conv_kwargs1, conv_op = nn.Conv2d, norm_op = nn.InstanceNorm2d, dropout_op = nn.Dropout2d)
+        )
+        conv_kwargs1 = {'kernel_size' : 3, 'padding' : 1, 'stride' : 2}
+        conv_kwargs2 = {'kernel_size' : 3, 'padding' : 1, 'stride' : 1}
+        self.encoder2 = nn.Sequential(
+            ConvDropoutNormNonlin(encoder_channel1, encoder_channel2, conv_kwargs= conv_kwargs1, conv_op = nn.Conv2d, norm_op = nn.InstanceNorm2d, dropout_op = nn.Dropout2d),
+            ConvDropoutNormNonlin(encoder_channel2, encoder_channel2, conv_kwargs= conv_kwargs2, conv_op = nn.Conv2d, norm_op = nn.InstanceNorm2d, dropout_op = nn.Dropout2d)
+        )
+        self.encoder3 = nn.Sequential(
+            ConvDropoutNormNonlin(encoder_channel2, encoder_channel3, conv_kwargs= conv_kwargs1, conv_op = nn.Conv2d, norm_op = nn.InstanceNorm2d, dropout_op = nn.Dropout2d),
+            ConvDropoutNormNonlin(encoder_channel3, encoder_channel3, conv_kwargs= conv_kwargs2, conv_op = nn.Conv2d, norm_op = nn.InstanceNorm2d, dropout_op = nn.Dropout2d)
+        )
+        self.encoder4 = nn.Sequential(
+            ConvDropoutNormNonlin(encoder_channel3, encoder_channel4, conv_kwargs= conv_kwargs1, conv_op = nn.Conv2d, norm_op = nn.InstanceNorm2d, dropout_op = nn.Dropout2d),
+            ConvDropoutNormNonlin(encoder_channel4, encoder_channel4, conv_kwargs= conv_kwargs2, conv_op = nn.Conv2d, norm_op = nn.InstanceNorm2d, dropout_op = nn.Dropout2d)
+        )
+        self.encoder5 = nn.Sequential(
+            ConvDropoutNormNonlin(encoder_channel4, encoder_channel5, conv_kwargs= conv_kwargs1, conv_op = nn.Conv2d, norm_op = nn.InstanceNorm2d, dropout_op = nn.Dropout2d),
+            ConvDropoutNormNonlin(encoder_channel5, encoder_channel5, conv_kwargs= conv_kwargs2, conv_op = nn.Conv2d, norm_op = nn.InstanceNorm2d, dropout_op = nn.Dropout2d)
+        )
+    
+
+
+class Deep_Supervision_UNet3d_Narrow(nn.Module):
     def __init__(self, n_modalities = 4, final_output_channels = 5, patch_size = (128, 128, 128), is_binary = False):
         super().__init__()
         self.patch_size = np.array(patch_size)
@@ -79,7 +114,7 @@ class Deep_Supervision_UNet3d_V2(nn.Module):
             ConvDropoutNormNonlin(256, 256, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
         )
         self.encoder5 = nn.Sequential(
-            ConvDropoutNormNonlin(256, 320, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
+            ConvDropoutNormNonlin(256, 320, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d),
             ConvDropoutNormNonlin(320, 320, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
         )
 
@@ -88,7 +123,7 @@ class Deep_Supervision_UNet3d_V2(nn.Module):
         transp_kwargs = {'kernel_size' : 2, 'stride' : 1, 'stride' : 2}
 
 
-        self.decoder1_trans = ConvDropoutNormNonlin(320, 256, conv_kwargs= transp_kwargs, conv_op = nn.ConvTranspose3d)
+        self.decoder1_trans = ConvDropoutNormNonlin(320, 256, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d)
         self.decoder1_conv = nn.Sequential(
             ConvDropoutNormNonlin(256*2, 256, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
             ConvDropoutNormNonlin(256, 256, conv_kwargs = conv_kwargs2, conv_op = nn.Conv3d)
@@ -109,25 +144,18 @@ class Deep_Supervision_UNet3d_V2(nn.Module):
             ConvDropoutNormNonlin(32, 32, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
         )
 
+
         self.final_layer1 = nn.Sequential(
-            nn.Conv3d(32, final_output_channels, 3, 1, 1),
-            nn.InstanceNorm3d(final_output_channels),
-            nn.LeakyReLU(),
+            nn.Conv3d(32, final_output_channels, 3, 1, 1)
             )
         self.final_layer2 = nn.Sequential(
-            nn.Conv3d(64, final_output_channels, 3, 1, 1),
-            nn.InstanceNorm3d(final_output_channels),
-            nn.LeakyReLU(),
+            nn.Conv3d(64, final_output_channels, 3, 1, 1)
             )
         self.final_layer3 = nn.Sequential(
-            nn.Conv3d(128, final_output_channels, 3, 1, 1),
-            nn.InstanceNorm3d(final_output_channels),
-            nn.LeakyReLU(),
+            nn.Conv3d(128, final_output_channels, 3, 1, 1)
             )
         self.final_layer4 = nn.Sequential(
-            nn.Conv3d(256, final_output_channels, 3, 1, 1),
-            nn.InstanceNorm3d(final_output_channels),
-            nn.LeakyReLU(),
+            nn.Conv3d(256, final_output_channels, 3, 1, 1)
             )
 
         self.upsample4 = nn.Upsample(scale_factor=8, mode = 'trilinear')
@@ -142,7 +170,6 @@ class Deep_Supervision_UNet3d_V2(nn.Module):
         x3 = self.encoder3(x2)
         x4 = self.encoder4(x3)
         x5 = self.encoder5(x4)
-
         x4_ = self.decoder1_conv(torch.cat((x4, self.decoder1_trans(x5)), dim = 1))
         x3_ = self.decoder2_conv(torch.cat((x3, self.decoder2_trans(x4_)), dim = 1))
         x2_ = self.decoder3_conv(torch.cat((x2, self.decoder3_trans(x3_)), dim = 1))
@@ -230,36 +257,16 @@ class Deep_Supervision_UNet3d(nn.Module):
         )
 
         self.final_layer1 = nn.Sequential(
-            nn.Conv3d(32, 16, 3, 1, 1),
-            nn.InstanceNorm3d(16),
-            nn.LeakyReLU(),
-            nn.Conv3d(16, final_output_channels, 3, 1, 1),
-            nn.InstanceNorm3d(final_output_channels),
-            nn.LeakyReLU(),
+            nn.Conv3d(32, final_output_channels, 3, 1, 1)
             )
         self.final_layer2 = nn.Sequential(
-            nn.Conv3d(64, 32, 3, 1, 1),
-            nn.InstanceNorm3d(32),
-            nn.LeakyReLU(),
-            nn.Conv3d(32, final_output_channels, 3, 1, 1),
-            nn.InstanceNorm3d(final_output_channels),
-            nn.LeakyReLU(),
+            nn.Conv3d(64, final_output_channels, 3, 1, 1)
             )
         self.final_layer3 = nn.Sequential(
-            nn.Conv3d(128, 64, 3, 1, 1),
-            nn.InstanceNorm3d(64),
-            nn.LeakyReLU(),
-            nn.Conv3d(64, final_output_channels, 3, 1, 1),
-            nn.InstanceNorm3d(final_output_channels),
-            nn.LeakyReLU(),
+            nn.Conv3d(128, final_output_channels, 3, 1, 1)
             )
         self.final_layer4 = nn.Sequential(
-            nn.Conv3d(256, 128, 3, 1, 1),
-            nn.InstanceNorm3d(128),
-            nn.LeakyReLU(),
-            nn.Conv3d(128, final_output_channels, 3, 1, 1),
-            nn.InstanceNorm3d(final_output_channels),
-            nn.LeakyReLU(),
+            nn.Conv3d(256, final_output_channels, 3, 1, 1)
             )
 
         self.upsample4 = nn.Upsample(scale_factor=8, mode = 'trilinear')
@@ -343,65 +350,76 @@ class Interactive_Model(nn.Module):
 
         return torch.sigmoid(x)
 
+
+
+
 class Daegu_Model10(nn.Module):
-    def __init__(self, patch_size):
+    def __init__(self, patch_size, resnet_output = 4):
         super().__init__()
-        kwargs = {'sample_input_D':patch_size[0], 'sample_input_H':patch_size[1], 'sample_input_W':patch_size[2],'num_seg_classes':2}
+        resnet_output = resnet_output
+        out_tconv1 = resnet_output//2
+        out_tconv2 = out_tconv1//2
+        kwargs = {'sample_input_D':patch_size[0], 'sample_input_H':patch_size[1], 'sample_input_W':patch_size[2],'num_seg_classes':resnet_output}
         self.model = resnet10(**kwargs)
-        self.final_upsample = nn.Upsample(scale_factor=4, mode = 'trilinear')
+
+        self.upsample1 = nn.Sequential(nn.ConvTranspose3d(in_channels = resnet_output, out_channels= out_tconv1, kernel_size =3 , stride = 2, padding = 1, output_padding =1),
+                                       nn.InstanceNorm3d(out_tconv1),
+                                       nn.LeakyReLU())
+        
+        self.upsample2 = nn.Sequential(nn.ConvTranspose3d(in_channels = out_tconv1, out_channels= out_tconv2, kernel_size =3 , stride = 2, padding = 1, output_padding =1),
+                                       nn.InstanceNorm3d(out_tconv2),
+                                       nn.LeakyReLU())
+        
+        self.final_layer = nn.Sequential(nn.Conv3d(in_channels = out_tconv2, out_channels = 2, kernel_size = 3, stride = 1, padding = 1),
+                                   nn.InstanceNorm3d(2),
+                                   nn.LeakyReLU())
 
     def forward(self, X):
         X = self.model(X)
-        X = self.final_upsample(X)
+        X = self.upsample1(X)
+        X = self.upsample2(X)
+        X = self.final_layer(X)
         return torch.softmax(X, dim = -1)
 
 class Daegu_Model34(nn.Module):
-    def __init__(self, patch_size):
+    def __init__(self, patch_size, input_channel = 1, output_channel= 2):
         super().__init__()
-        kwargs = {'sample_input_D':patch_size[0], 'sample_input_H':patch_size[1], 'sample_input_W':patch_size[2],'num_seg_classes':2}
-        self.model = resnet34(**kwargs)
-        self.final_upsample = nn.Upsample(scale_factor=4, mode = 'trilinear')
+        out_channel1 = 1
+        out_channel2 = out_channel1 * 4
+
+        self.a_conv1 = nn.Sequential(nn.Conv3d(in_channels = input_channel, out_channels = out_channel1, kernel_size = 3, stride = 1, padding = 6, dilation = 6),
+                                   nn.InstanceNorm3d(out_channel1),
+                                   nn.LeakyReLU())
+        self.a_conv2 = nn.Sequential(nn.Conv3d(in_channels = input_channel, out_channels = out_channel1, kernel_size = 3, stride = 1, padding = 12, dilation = 12),
+                                   nn.InstanceNorm3d(out_channel1),
+                                   nn.LeakyReLU())
+        self.a_conv3 = nn.Sequential(nn.Conv3d(in_channels = input_channel, out_channels = out_channel1, kernel_size = 3, stride = 1, padding = 18, dilation = 18),
+                                   nn.InstanceNorm3d(out_channel1),
+                                   nn.LeakyReLU())
+        self.a_conv4 = nn.Sequential(nn.Conv3d(in_channels = input_channel, out_channels = out_channel1, kernel_size = 1, stride = 1, padding = 0, dilation = 1),
+                                   nn.InstanceNorm3d(out_channel1),
+                                   nn.LeakyReLU())
+
+        self.a_conv2_1 = nn.Sequential(nn.Conv3d(in_channels = out_channel2, out_channels = out_channel2, kernel_size = 1, stride = 1, padding = 0, dilation = 1),
+                                   nn.InstanceNorm3d(out_channel2),
+                                   nn.LeakyReLU())
+
+        self.last_layer = nn.Sequential(nn.Conv3d(in_channels = out_channel2, out_channels = output_channel, kernel_size = 1, stride = 1, padding = 0, dilation = 1),
+                                   nn.InstanceNorm3d(output_channel),
+                                   nn.LeakyReLU())
+
 
     def forward(self, X):
-        X = self.model(X)
-        X = self.final_upsample(X)
+        X1 = self.a_conv1(X)
+        X2 = self.a_conv2(X)
+        X3 = self.a_conv3(X)
+        X4 = self.a_conv4(X)
+
+        X = torch.cat((X1,X2,X3,X4), dim = 1)
+        X = self.a_conv2_1(X)
+        X = self.last_layer(X)
         return torch.softmax(X, dim = -1)
 
-class Daegu_Model101(nn.Module):
-    def __init__(self, patch_size):
-        super().__init__()
-        kwargs = {'sample_input_D':patch_size[0], 'sample_input_H':patch_size[1], 'sample_input_W':patch_size[2],'num_seg_classes':2}
-        self.model = resnet101(**kwargs)
-        self.final_upsample = nn.Upsample(scale_factor=4, mode = 'trilinear')
-
-    def forward(self, X):
-        X = self.model(X)
-        X = self.final_upsample(X)
-        return torch.softmax(X, dim = -1)
-
-class Daegu_Model200(nn.Module):
-    def __init__(self, patch_size):
-        super().__init__()
-        kwargs = {'sample_input_D':patch_size[0], 'sample_input_H':patch_size[1], 'sample_input_W':patch_size[2],'num_seg_classes':2}
-        self.model = resnet200(**kwargs)
-        self.final_upsample = nn.Upsample(scale_factor=4, mode = 'trilinear')
-
-    def forward(self, X):
-        X = self.model(X)
-        X = self.final_upsample(X)
-        return torch.softmax(X, dim = -1)
-
-class Daegu_Model50(nn.Module):
-    def __init__(self, patch_size):
-        super().__init__()
-        kwargs = {'sample_input_D':patch_size[0], 'sample_input_H':patch_size[1], 'sample_input_W':patch_size[2],'num_seg_classes':2}
-        self.model = resnet50(**kwargs)
-        self.final_upsample = nn.Upsample(scale_factor=4, mode = 'trilinear')
-
-    def forward(self, X):
-        X = self.model(X)
-        X = self.final_upsample(X)
-        return torch.softmax(X, dim = -1)
 
 
 
