@@ -15,13 +15,14 @@ from ingradient_library.sampling import *
 
 class Trainer(object):
     def __init__(self, tr_dataloader, model,  optimizer, losses, regulizers = [], is_deep_supervision = False,
-                deep_supervision_weight = 0.5, n_epoch = 1000, val_dataloader = None, save_path = None, scheduler = None):
+                deep_supervision_weight = 0.5, n_epoch = 1000, val_dataloader = None, save_path = None, scheduler = None, visualization = False):
  
         self.model = model
+        self.visualization = visualization
         self.optimizer = optimizer
         self.n_epoch = n_epoch
         self.save_path = save_path
-        self.visualization = None
+        self.visualization = visualization
         self.tr_dl = tr_dataloader
         self.val_dl = val_dataloader
         self.scheduler = scheduler
@@ -32,7 +33,7 @@ class Trainer(object):
         if len(self.regulizers) == 0:
             self.regulizers = [1] * len(self.losses)
             
-    def calculate_loss(self, output, target, writer, epoch):
+    def calculate_loss(self, output, target, writer, epoch, is_val = False):
         if self.is_deep_supervision == True:
             batch_size = output.shape[0]
             deep_supervision_size = output.shape[1]
@@ -68,7 +69,10 @@ class Trainer(object):
             temp = loss(output, target).contiguous() * torch.tensor(self.regulizers[i]).float().to(output.device.index)
             for _ in range(len(temp.shape)-1):
                 temp = temp.mean(-1)
-            writer.add_scalar('Iteration\Loss_'+ str(type(loss)), temp[0], epoch) #(batch, -1) 로 나오게끔 resize
+            if is_val:
+                writer.add_scalar('Iteration\Val_Loss_'+ str(type(loss)), temp[0], epoch) #(batch, -1) 로 나오게끔 resize
+            else:
+                writer.add_scalar('Iteration\Train_Loss_'+ str(type(loss)), temp[0], epoch) #(batch, -1) 로 나오게끔 resize
             if self.is_deep_supervision == True:
                 temp = temp.view(batch_size, deep_supervision_size, -1)
             if i == 0:
@@ -100,20 +104,25 @@ class Trainer(object):
 
     def run(self):
         writer = SummaryWriter()
+        current_iter = 0
+        val_iter = 0
         for e in range(self.n_epoch):
             self.tr_dl.new_epoch()
             n_iter = 0
             self.model.train()
-            
+            epoch_tr_loss = 0
+            epoch_tr_div = 0
+            epoch_val_loss = 0
+            epoch_val_div = 0
             while not self.tr_dl.is_end():
+                current_iter += 1
                 n_iter += 1
                 images, seg = self.tr_dl.generate_train_batch()
                 if isinstance(self.optimizer, SAMSGD):
                     def closure():
                         self.optimizer.zero_grad()
                         output = self.model(images)
-                        current_iter = (self.tr_dl.current_index* self.tr_dl.num_iteration + self.tr_dl.now_iter) /(self.tr_dl.num_iteration*len(self.tr_dl.dataset))
-                        loss = self.calculate_loss(output, seg, writer, e + current_iter)
+                        loss = self.calculate_loss(output, seg, writer, current_iter)
                         loss.backward()
                         return loss
                     self.optimizer.step(closure)
@@ -121,26 +130,31 @@ class Trainer(object):
                 else:
                     output = self.model(images)
                     self.optimizer.zero_grad()
-                    current_iter = (self.tr_dl.current_index* self.tr_dl.num_iteration + self.tr_dl.now_iter) /(self.tr_dl.num_iteration*len(self.tr_dl.dataset))
-                    loss = self.calculate_loss(output, seg, writer, e + current_iter)
+                    loss = self.calculate_loss(output, seg, writer, current_iter)
                     loss.backward()
                     self.optimizer.step()
-                
-                if self.tr_dl.current_index < 10 and not isinstance(self.optimizer, SAMSGD):
+
+                epoch_tr_loss += loss.item()
+                epoch_tr_div += 1
+
+                if self.tr_dl.current_index < 3 and not isinstance(self.optimizer, SAMSGD) and self.visualization:
                     print("###########", self.tr_dl.current_index, "###########")
+                    n_classes = 1
                     if not self.is_deep_supervision:
                         print('output')
-                        plt.imshow(torch.argmax(output[0,:,output.shape[2]//2], dim = 0).detach().cpu(), vmax =1 , vmin= 0)
+                        n_classes = output.shape[2]
+                        plt.imshow(torch.argmax(output[0,:,:,:,output.shape[-1]//2], dim = 0).detach().cpu(), vmax =  n_classes, vmin= 0)
                         plt.show()
                     elif not isinstance(self.optimizer, SAMSGD):
                         print('output')
-                        plt.imshow(torch.argmax(output[0,0,:,output.shape[2]//2], dim = 0).detach().cpu(), vmax =1 , vmin= 0)
+                        n_classes = output.shape[1]
+                        plt.imshow(torch.argmax(output[0,0,:,:,:,output.shape[-1]//2], dim = 0).detach().cpu(), vmax = n_classes, vmin= 0)
                         plt.show()
                     print('images')
-                    plt.imshow(images[0,0,output.shape[2]//2].detach().cpu())
+                    plt.imshow(images[0,0,:,:,output.shape[-1]//2].detach().cpu())
                     plt.show()
                     print('ground_truth')
-                    plt.imshow(seg[0,output.shape[2]//2].detach().cpu(), vmax =1 , vmin= 0)
+                    plt.imshow(seg[0,:,:,output.shape[-1]//2].detach().cpu(), vmax = n_classes , vmin= 0)
                     plt.show()
 
 
@@ -150,15 +164,18 @@ class Trainer(object):
                 n_iter = 0
                 self.model.eval()
                 while not self.val_dl.is_end():
+                    val_iter +=1
                     n_iter +=1
                     with torch.no_grad():
                         images, seg = self.val_dl.generate_train_batch()
                         output = self.model(images)
-                        current_iter = (self.val_dl.current_index* self.val_dl.num_iteration + self.val_dl.now_iter) /(self.val_dl.num_iteration*len(self.val_dl.dataset))
-                        loss = self.calculate_loss(output, seg, writer, e + current_iter)
-                        if self.visualization != None:
-                            if self.val_dl.current_index == 0:
-                                self.visualization(output, seg)
+                        loss = self.calculate_loss(output, seg, writer, val_iter, True)
+                        epoch_val_loss += loss.item()
+                        epoch_val_div += 1
+
+            writer.add_scalar('epoch\Loss_train', epoch_tr_loss/epoch_tr_div , e) #(batch, -1) 로 나오게끔 resize
+            if self.val_dl != None:
+                writer.add_scalar('epoch\Loss_val', epoch_val_loss/epoch_val_div , e) #(batch, -1) 로 나오게끔 resize
 
             if self.scheduler != None: 
                 self.scheduler.step()
